@@ -24,7 +24,9 @@
 #include <math.h>
 #include <string.h>
 
-#include "pango.h"
+#include "pango-types.h"
+#include "pango-font.h"
+#include "pango-fontmap.h"
 
 /**
  * pango_font_description_copy:
@@ -365,6 +367,35 @@ pango_font_description_to_string (const PangoFontDescription  *desc)
   return str;
 }
 
+/**
+ * pango_font_description_to_filename:
+ * @desc: a #PangoFontDescription
+ * 
+ * Create a filename representation of a font description. The
+ * filename is identical to the result from calling
+ * pango_font_description_to_string(), but with underscores instead of
+ * characters that are untypical in filenames, and in lower case only.
+ *
+ * Return value: a new string that must be freed with g_free().
+ **/
+char *
+pango_font_description_to_filename (const PangoFontDescription  *desc)
+{
+  char *result = pango_font_description_to_string (desc);
+  char *p;
+
+  g_strdown (result);
+  p = result;
+  while (*p)
+    {
+      if (strchr ("-+_.", *p) == NULL && !isalnum (*p))
+	*p = '_';
+      p++;
+    }
+
+  return result;
+}  
+  
 GType
 pango_font_get_type (void)
 {
@@ -393,6 +424,22 @@ pango_font_get_type (void)
   return object_type;
 }
 
+/**
+ * pango_font_describe:
+ * @font: a #PangoFont
+ * 
+ * Return a description of the font.
+ * 
+ * Return value: a newly allocated #PangoFontDescription object.
+ **/
+PangoFontDescription *
+pango_font_describe (PangoFont      *font)
+{
+  g_return_val_if_fail (font != NULL, NULL);
+
+  return PANGO_FONT_GET_CLASS (font)->describe (font);
+}
+
 /**
  * pango_font_get_coverage:
  * @font: a #PangoFont
@@ -427,9 +474,13 @@ pango_font_find_shaper (PangoFont      *font,
 			const char     *lang,
 			guint32         ch)
 {
+  PangoEngineShape* shaper;
+  
   g_return_val_if_fail (font != NULL, NULL);
 
-  return PANGO_FONT_GET_CLASS (font)->find_shaper (font, lang, ch);
+  shaper = PANGO_FONT_GET_CLASS (font)->find_shaper (font, lang, ch);
+
+  return shaper;
 }
 
 /**
