@@ -89,17 +89,6 @@ struct _PangoFT2FamilyEntry
   GSList *font_entries;
 };
 
-struct _PangoFT2FontEntry
-{
-  FT_Open_Args **open_args;
-  FT_Long *face_indices;
-  int n_fonts;
-  PangoFontDescription description;
-  PangoCoverage *coverage;
-
-  GSList *cached_fonts;
-};
-
 static GType      pango_ft2_font_map_get_type      (void);
 
 static void       pango_ft2_font_map_init          (PangoFT2FontMap             *fontmap);
@@ -252,7 +241,7 @@ pango_ft2_is_font_file (const char *name)
   return FALSE;
 }
 
-static void
+static gboolean
 pango_ft2_scan_directory (const char      *path,
 			  PangoFT2FontMap *ft2fontmap)
 {
@@ -262,11 +251,12 @@ pango_ft2_scan_directory (const char      *path,
   FT_Face face;
   FT_Error error;
   int i;
+  gboolean found_font = FALSE;
 
   dir = opendir (path);
   if (!dir)
     /* Don't warn; it's OK to have nonexistent entries in the font path */
-    return;
+    return FALSE;
 
   while ((entry = readdir (dir)) != NULL)
     {
@@ -284,7 +274,10 @@ pango_ft2_scan_directory (const char      *path,
 	  else
 	    {
 	      if (face->face_flags & FT_FACE_FLAG_SCALABLE)
-		pango_ft2_insert_face (ft2fontmap, face, fullname, 0);
+		{
+		  pango_ft2_insert_face (ft2fontmap, face, fullname, 0);
+		  found_font = TRUE;
+		}
 	      
 	      for (i = 1; i < face->num_faces; i++)
 		{
@@ -297,7 +290,10 @@ pango_ft2_scan_directory (const char      *path,
 		    g_warning ("Error loading font %d from '%s': %s",
 			       i, fullname, pango_ft2_ft_strerror (error));
 		  else if (face->face_flags & FT_FACE_FLAG_SCALABLE)
-		    pango_ft2_insert_face (ft2fontmap, face, fullname, i);
+		    {
+		      pango_ft2_insert_face (ft2fontmap, face, fullname, i);
+		      found_font = TRUE;
+		    }
 		}
 	      error = FT_Done_Face (face);
 	      if (error != FT_Err_Ok)
@@ -308,6 +304,7 @@ pango_ft2_scan_directory (const char      *path,
       g_free (fullname);
     }
   closedir (dir);
+  return found_font;
 }
 
 PangoFontMap *
@@ -315,14 +312,15 @@ pango_ft2_font_map_for_display (void)
 {
   char **tmp_list;
   FT_Error error;
+  gboolean read_font;
 
   /* Make sure that the type system is initialized */
-  g_type_init();
+  g_type_init (0);
   
   if (pango_ft2_global_fontmap != NULL)
     return PANGO_FONT_MAP (pango_ft2_global_fontmap);
 
-  pango_ft2_global_fontmap = (PangoFT2FontMap *)g_type_create_instance (PANGO_TYPE_FT2_FONT_MAP);
+  pango_ft2_global_fontmap = (PangoFT2FontMap *)g_object_new (PANGO_TYPE_FT2_FONT_MAP, NULL);
   
   error = FT_Init_FreeType (&pango_ft2_global_fontmap->library);
   if (error != FT_Err_Ok)
@@ -337,12 +335,16 @@ pango_ft2_font_map_for_display (void)
   
   tmp_list = pango_ft2_font_directories;
 
+  read_font = FALSE;
   while (*tmp_list)
     {
-      pango_ft2_scan_directory ((const char *) *tmp_list, pango_ft2_global_fontmap);
+      read_font |= pango_ft2_scan_directory ((const char *) *tmp_list, pango_ft2_global_fontmap);
       tmp_list++;
     }
 
+  if (!read_font)
+    g_warning ("No fonts found by pangft2. Things will probably not work");
+  
   pango_ft2_font_map_read_aliases (pango_ft2_global_fontmap);
 
   return PANGO_FONT_MAP (pango_ft2_global_fontmap);
@@ -531,6 +533,8 @@ pango_ft2_font_map_load_font (PangoFontMap               *fontmap,
   g_strdown (name);
 
   family_entry = g_hash_table_lookup (ft2fontmap->families, name);
+  g_free (name);
+  
   if (family_entry)
     {
       PangoFT2FontEntry *best_match = NULL;
@@ -594,11 +598,10 @@ pango_ft2_font_map_load_font (PangoFontMap               *fontmap,
 	}
     }
 
-  g_free (name);
   return result;
 }
 
-static void
+static gboolean
 pango_ft2_font_map_read_alias_file (PangoFT2FontMap *ft2fontmap,
 				    const char      *filename)
 {
@@ -608,6 +611,7 @@ pango_ft2_font_map_read_alias_file (PangoFT2FontMap *ft2fontmap,
   int i;
   PangoFT2FontEntry *font_entry = NULL;
   gchar **faces;
+  gboolean ret_val = FALSE;
 
   infile = fopen (filename, "r");
   if (infile)
@@ -697,6 +701,8 @@ pango_ft2_font_map_read_alias_file (PangoFT2FontMap *ft2fontmap,
 		}
 	    }
 
+	  g_strfreev (faces);
+	  
 	  /* Insert the font entry into our structures */
 
 	  family_entry = pango_ft2_get_family_entry (ft2fontmap, font_entry->description.family_name);
@@ -712,6 +718,7 @@ pango_ft2_font_map_read_alias_file (PangoFT2FontMap *ft2fontmap,
       if (ferror (infile))
 	g_warning ("Error reading file '%s': %s", filename, g_strerror(errno));
 
+      ret_val = TRUE;
       goto out;
 
     error:
@@ -735,6 +742,7 @@ pango_ft2_font_map_read_alias_file (PangoFT2FontMap *ft2fontmap,
       fclose (infile);
     }
 
+  return ret_val;
 }
 
 static void
@@ -745,6 +753,7 @@ pango_ft2_font_map_read_aliases (PangoFT2FontMap *ft2fontmap)
   char *home;
   char *tmp_str;
   int n;
+  gboolean read_aliasfile;
 
   if (!files_str)
     {
@@ -769,8 +778,14 @@ pango_ft2_font_map_read_aliases (PangoFT2FontMap *ft2fontmap)
   while (files[n])
     n++;
 
+
+  read_aliasfile = FALSE;
+  
   while (n-- > 0)
-    pango_ft2_font_map_read_alias_file (ft2fontmap, files[n]);
+    read_aliasfile |= pango_ft2_font_map_read_alias_file (ft2fontmap, files[n]);
+
+  if (!read_aliasfile)
+    g_warning ("Didn't read any pango ft2 fontalias file. Things will probably not work.");
 
   g_strfreev (files);
   g_free (files_str);
@@ -886,7 +901,7 @@ pango_ft2_insert_face (PangoFT2FontMap *ft2fontmap,
 	  g_free (description->family_name);
 	  g_free (description);
 #if 0
-	  PING ((" family and description matched (!)\n"));
+	  PING ((" family and description matched (!)"));
 #endif
 	  return;
 	}
@@ -949,6 +964,12 @@ pango_ft2_font_entry_get_coverage (PangoFT2FontEntry *entry,
   PangoCoverageLevel font_level;
   PangoMapEntry *map_entry;
   GHashTable *coverage_hash;
+  PangoFontDescription *description;
+  FILE *cache_file;
+  gchar *cache_file_name;
+  gchar *font_as_filename;
+  guchar *buf;
+  int buflen;
 
   if (entry)
     if (entry->coverage)
@@ -957,36 +978,68 @@ pango_ft2_font_entry_get_coverage (PangoFT2FontEntry *entry,
 	return entry->coverage;
       }
 
-  result = pango_coverage_new ();
+  description = pango_font_describe (font);
+  font_as_filename = pango_font_description_to_filename (description);
+  cache_file_name = g_strconcat (pango_get_sysconf_subdirectory (),
+				 G_DIR_SEPARATOR_S "cache.ft2" G_DIR_SEPARATOR_S,
+				 font_as_filename,
+				 ".",
+				 lang ? lang : "",
+				 NULL);
+  g_free (font_as_filename);
+  pango_font_description_free (description);
 
-  coverage_hash = g_hash_table_new (g_str_hash, g_str_equal);
-  
-  shape_map = pango_ft2_get_shaper_map (lang);
-
-  for (ch = 0; ch < 65536; ch++)
+  PING (("trying to load %s", cache_file_name));
+  result = NULL;
+  if (g_file_get_contents (cache_file_name, &buf, &buflen, NULL))
     {
-      map_entry = pango_map_get_entry (shape_map, ch);
-      if (map_entry->info)
-	{
-	  coverage = g_hash_table_lookup (coverage_hash, map_entry->info->id);
-	  if (!coverage)
-	    {
-	      PangoEngineShape *engine = (PangoEngineShape *)pango_map_get_engine (shape_map, ch);
-	      coverage = engine->get_coverage (font, lang);
-	      g_hash_table_insert (coverage_hash, map_entry->info->id, coverage);
-	    }
-	  
-	  font_level = pango_coverage_get (coverage, ch);
-	  if (font_level == PANGO_COVERAGE_EXACT && !map_entry->is_exact)
-	    font_level = PANGO_COVERAGE_APPROXIMATE;
-
-	  if (font_level != PANGO_COVERAGE_NONE)
-	    pango_coverage_set (result, ch, font_level);
-	}
+      result = pango_coverage_from_bytes (buf, buflen);
+      g_free (buf);
     }
 
-  g_hash_table_foreach (coverage_hash, free_coverages_foreach, NULL);
-  g_hash_table_destroy (coverage_hash);
+  if (!result)
+    {
+      result = pango_coverage_new ();
+
+      coverage_hash = g_hash_table_new (g_str_hash, g_str_equal);
+  
+      shape_map = pango_ft2_get_shaper_map (lang);
+
+      for (ch = 0; ch < 65536; ch++)
+	{
+	  map_entry = pango_map_get_entry (shape_map, ch);
+	  if (map_entry->info)
+	    {
+	      coverage = g_hash_table_lookup (coverage_hash, map_entry->info->id);
+	      if (!coverage)
+		{
+		  PangoEngineShape *engine = (PangoEngineShape *)pango_map_get_engine (shape_map, ch);
+		  coverage = engine->get_coverage (font, lang);
+		  g_hash_table_insert (coverage_hash, map_entry->info->id, coverage);
+		}
+	      
+	      font_level = pango_coverage_get (coverage, ch);
+	      if (font_level == PANGO_COVERAGE_EXACT && !map_entry->is_exact)
+		font_level = PANGO_COVERAGE_APPROXIMATE;
+
+	      if (font_level != PANGO_COVERAGE_NONE)
+		pango_coverage_set (result, ch, font_level);
+	    }
+	}
+
+      g_hash_table_foreach (coverage_hash, free_coverages_foreach, NULL);
+      g_hash_table_destroy (coverage_hash);
+
+      cache_file = fopen (cache_file_name, "wb");
+      if (cache_file)
+	{
+	  pango_coverage_to_bytes (result, &buf, &buflen);
+	  PING (("saving %d bytes to %s", buflen, cache_file_name));
+	  fwrite (buf, buflen, 1, cache_file);
+	  fclose (cache_file);
+	  g_free (buf);
+	}
+    }
 
   if (entry)
     {
@@ -994,6 +1047,8 @@ pango_ft2_font_entry_get_coverage (PangoFT2FontEntry *entry,
       pango_coverage_ref (result);
     }
 
+  g_free (cache_file_name);
+
   return result;
 }
 
